# THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT MANUALLY.

from typing import Dict, List, Optional, Sequence, Set, Tuple, Union

from collections import OrderedDict

import ast
import docutils.nodes

Document = docutils.nodes.document

__version__ = ...  # type: str
SIG_FIELD = ...  # type: str
SIG_COMMENT = ...  # type: str
SIG_ALIAS = ...  # type: str
DECORATORS = ...  # type: Set[str]

def get_fields(
    node: Document, fields_tag: Optional[str] = ...
) -> Dict[str, str]: ...
def extract_signature(docstring: str) -> Optional[str]: ...
def get_signature(
    node: Union[ast.FunctionDef, ast.AsyncFunctionDef, ast.ClassDef]
) -> Optional[str]: ...
def split_parameter_types(parameters: str) -> List[str]: ...
def parse_signature(signature: str) -> Tuple[List[str], str, Set[str]]: ...

class StubNode:
    variables = ...  # type: List[VariableNode]
    children = ...  # type: List[Union[FunctionNode, ClassNode]]
    parent = ...  # type: Optional[StubNode]
    def __init__(self) -> None: ...
    def add_variable(self, node: VariableNode) -> None: ...
    def add_child(self, node: Union[FunctionNode, ClassNode]) -> None: ...
    def get_code(self) -> List[str]: ...

class VariableNode(StubNode):
    name = ...  # type: str
    type_ = ...  # type: str
    def __init__(self, name: str, type_: str) -> None: ...
    def get_code(self) -> List[str]: ...

class FunctionNode(StubNode):
    name = ...  # type: str
    parameters = ...  # type: Sequence[Tuple[str, str, bool]]
    rtype = ...  # type: str
    decorators = ...  # type: Sequence[str]
    _async = ...  # type: bool
    def __init__(
        self,
        name: str,
        parameters: Sequence[Tuple[str, str, bool]],
        rtype: str,
        decorators: Optional[Sequence[str]] = ...,
    ) -> None: ...
    def get_code(self) -> List[str]: ...

class ClassNode(StubNode):
    name = ...  # type: str
    bases = ...  # type: Sequence[str]
    signature = ...  # type: Optional[str]
    def __init__(
        self, name: str, bases: Sequence[str], signature: Optional[str] = ...
    ) -> None: ...
    def get_code(self) -> List[str]: ...

def get_aliases(lines: Sequence[str]) -> Dict[str, str]: ...

class StubGenerator(ast.NodeVisitor):
    root = ...  # type: StubNode
    imported_names = ...  # type: OrderedDict[str, str]
    defined_types = ...  # type: Set[str]
    required_types = ...  # type: Set[str]
    aliases = ...  # type: OrderedDict[str, str]
    _parents = ...  # type: List[StubNode]
    _code_lines = ...  # type: List[str]
    def __init__(self, source: str) -> None: ...
    def collect_aliases(self) -> None: ...
    def visit_ImportFrom(self, node: ast.ImportFrom) -> None: ...
    def visit_Assign(self, node: ast.Assign) -> None: ...
    def get_function_node(
        self, node: Union[ast.FunctionDef, ast.AsyncFunctionDef]
    ) -> FunctionNode: ...
    def visit_FunctionDef(self, node: ast.FunctionDef) -> None: ...
    def visit_AsyncFunctionDef(self, node: ast.AsyncFunctionDef) -> None: ...
    def visit_ClassDef(self, node: ast.ClassDef) -> None: ...
    @staticmethod
    def generate_import(module_: str, names: Set[str]) -> str: ...
    def generate_stub(self) -> str: ...

def get_stub(source: str) -> str: ...
def main(argv: Optional[List[str]] = ...) -> None: ...



# Copyright (C) 2016-2019 H. Turgut Uyar <uyar@tekir.org>
#
# pygenstub is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pygenstub is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pygenstub.  If not, see <http://www.gnu.org/licenses/>.

"""pygenstub is a utility for generating stub files from docstrings in source files.

It takes a source file as input and creates a stub file with the same base name
and the ``.pyi`` extension.

For more information, please refer to the documentation:
https://pygenstub.readthedocs.io/
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import ast
import inspect
import logging
import re
import sys
import textwrap
from argparse import ArgumentParser
from bisect import bisect
from collections import OrderedDict
from io import StringIO

from docutils.core import publish_doctree


__version__ = "1.2.3"  # sig: str


PY3 = sys.version_info >= (3, 0)

if not PY3:
    import __builtin__ as builtins
    from codecs import open
else:
    import builtins


# sigalias: Document = docutils.nodes.document


BUILTIN_TYPES = {k for k, t in builtins.__dict__.items() if isinstance(t, type)}
BUILTIN_TYPES.add("None")

SIG_FIELD = "sig"  # sig: str
SIG_COMMENT = "# sig:"  # sig: str
SIG_ALIAS = "# sigalias:"  # sig: str

DECORATORS = {"property", "staticmethod", "classmethod"}  # sig: Set[str]

LINE_LENGTH_LIMIT = 79
INDENT = 4 * " "

EDIT_WARNING = "THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT MANUALLY."

_RE_QUALIFIED_TYPES = re.compile(r"\w+(?:\.\w+)*")
_RE_COMMENT_IN_STRING = re.compile(r"""['"]\s*%(text)s\s*.*['"]""" % {"text": SIG_COMMENT})


_logger = logging.getLogger(__name__)


def get_fields(node, fields_tag="field_list"):
    """Get the field names and their values from a node.

    :sig: (Document, Optional[str]) -> Dict[str, str]
    :param node: Node to get the fields from.
    :param fields_tag: Tag of child node that contains the fields.
    :return: Names and values of fields.
    """
    fields_nodes = [c for c in node.children if c.tagname == fields_tag]
    if len(fields_nodes) == 0:
        return {}
    assert len(fields_nodes) == 1, "multiple nodes with tag " + fields_tag
    fields_node = fields_nodes[0]
    fields = [
        {f.tagname: f.rawsource.strip() for f in n.children}
        for n in fields_node.children
        if n.tagname == "field"
    ]
    return {f["field_name"]: f["field_body"] for f in fields}


def extract_signature(docstring):
    """Extract the signature from a docstring.

    :sig: (str) -> Optional[str]
    :param docstring: Docstring to extract the signature from.
    :return: Extracted signature, or ``None`` if there's no signature.
    """
    root = publish_doctree(docstring, settings_overrides={"report_level": 5})
    fields = get_fields(root)
    return fields.get(SIG_FIELD)


def get_signature(node):
    """Get the signature of a function or a class.

    :sig: (Union[ast.FunctionDef, ast.AsyncFunctionDef, ast.ClassDef]) -> Optional[str]
    :param node: Node to get the signature from.
    :return: Value of signature field in node docstring, or ``None`` if there's no signature.
    """
    docstring = ast.get_docstring(node)
    if docstring is None:
        return None
    return extract_signature(docstring)


def split_parameter_types(parameters):
    """Split a parameter types declaration into individual types.

    The input is the left hand side of a signature (the part before the arrow),
    excluding the parentheses.

    :sig: (str) -> List[str]
    :param parameters: Comma separated parameter types.
    :return: Parameter types.
    """
    if parameters == "":
        return []

    # only consider the top level commas, ignore the ones in []
    commas = []
    bracket_depth = 0
    for i, char in enumerate(parameters):
        if (char == ",") and (bracket_depth == 0):
            commas.append(i)
        elif char == "[":
            bracket_depth += 1
        elif char == "]":
            bracket_depth -= 1

    types = []
    last_i = 0
    for i in commas:
        types.append(parameters[last_i:i].strip())
        last_i = i + 1
    else:
        types.append(parameters[last_i:].strip())
    return types


def parse_signature(signature):
    """Parse a signature into its input and return parameter types.

    This will also collect the types that are required by any of the input
    and return types.

    :sig: (str) -> Tuple[List[str], str, Set[str]]
    :param signature: Signature to parse.
    :return: Input parameter types, return type, and all required types.
    """
    if " -> " not in signature:
        # signature comment: no parameters, treat variable type as return type
        param_types, return_type = None, signature.strip()
    else:
        lhs, return_type = [s.strip() for s in signature.split(" -> ")]
        csv = lhs[1:-1].strip()  # remove the parentheses around the parameter type list
        param_types = split_parameter_types(csv)
    requires = set(_RE_QUALIFIED_TYPES.findall(signature))
    return param_types, return_type, requires


class StubNode:
    """A node in a stub tree."""

    def __init__(self):
        """Initialize this stub node.

        :sig: () -> None
        """
        self.variables = []  # sig: List[VariableNode]
        self.children = []  # sig: List[Union[FunctionNode, ClassNode]]
        self.parent = None  # sig: Optional[StubNode]

    def add_variable(self, node):
        """Add a variable node to this node.

        :sig: (VariableNode) -> None
        :param node: Variable node to add.
        """
        self.variables.append(node)
        node.parent = self

    def add_child(self, node):
        """Add a function/method or class node to this node.

        :sig: (Union[FunctionNode, ClassNode]) -> None
        :param node: Function or class node to add.
        """
        self.children.append(node)
        node.parent = self

    def get_code(self):
        """Get the stub code for this node.

        The stub code for a node consists of the type annotations of its variables,
        followed by the prototypes of its functions/methods and classes.

        :sig: () -> List[str]
        :return: Lines of stub code for this node.
        """
        stub = []
        for child in self.variables:
            stub.extend(child.get_code())
        if (
            (len(self.variables) > 0)
            and (len(self.children) > 0)
            and (not isinstance(self, ClassNode))
        ):
            stub.append("")
        for child in self.children:
            stub.extend(child.get_code())
        return stub


class VariableNode(StubNode):
    """A node representing an assignment in a stub tree."""

    def __init__(self, name, type_):
        """Initialize this variable node.

        :sig: (str, str) -> None
        :param name: Name of variable that is being assigned to.
        :param type_: Type of variable.
        """
        if not PY3:
            StubNode.__init__(self)
        else:
            super().__init__()
        self.name = name  # sig: str
        self.type_ = type_  # sig: str

    def get_code(self):
        """Get the type annotation for this variable.

        :sig: () -> List[str]
        :return: Lines of stub code for this variable.
        """
        return ["%(n)s = ...  # type: %(t)s" % {"n": self.name, "t": self.type_}]


class FunctionNode(StubNode):
    """A node representing a function in a stub tree."""

    def __init__(self, name, parameters, rtype, decorators=None):
        """Initialize this function node.

        The parameters have to given as a list of triples where each item specifies
        the name of the parameter, its type, and whether it has a default value or not.

        :sig: (str, Sequence[Tuple[str, str, bool]], str, Optional[Sequence[str]]) -> None
        :param name: Name of function.
        :param parameters: List of parameter triples (name, type, has_default).
        :param rtype: Type of return value.
        :param decorators: Decorators of function.
        """
        if not PY3:
            StubNode.__init__(self)
        else:
            super().__init__()
        self.name = name  # sig: str
        self.parameters = parameters  # sig: Sequence[Tuple[str, str, bool]]
        self.rtype = rtype  # sig: str
        self.decorators = decorators if decorators is not None else []  # sig: Sequence[str]

        self._async = False  # sig: bool

    def get_code(self):
        """Get the stub code for this function.

        :sig: () -> List[str]
        :return: Lines of stub code for this function.
        """
        stub = []

        for deco in self.decorators:
            if (deco in DECORATORS) or deco.endswith(".setter"):
                stub.append("@" + deco)

        parameters = []
        for name, type_, has_default in self.parameters:
            decl = "%(n)s%(t)s%(d)s" % {
                "n": name,
                "t": ": " + type_ if type_ else "",
                "d": " = ..." if has_default else "",
            }
            parameters.append(decl)

        slots = {
            "a": "async " if self._async else "",
            "n": self.name,
            "p": ", ".join(parameters),
            "r": self.rtype,
        }

        prototype = "%(a)sdef %(n)s(%(p)s) -> %(r)s: ..." % slots
        if len(prototype) <= LINE_LENGTH_LIMIT:
            stub.append(prototype)
        elif len(INDENT + slots["p"]) <= LINE_LENGTH_LIMIT:
            stub.append("%(a)sdef %(n)s(" % slots)
            stub.append(INDENT + slots["p"])
            stub.append(") -> %(r)s: ..." % slots)
        else:
            stub.append("%(a)sdef %(n)s(" % slots)
            for param in parameters:
                stub.append(INDENT + param + ",")
            stub.append(") -> %(r)s: ..." % slots)

        return stub


class ClassNode(StubNode):
    """A node representing a class in a stub tree."""

    def __init__(self, name, bases, signature=None):
        """Initialize this class node.

        :sig: (str, Sequence[str], Optional[str]) -> None
        :param name: Name of class.
        :param bases: Base classes of class.
        :param signature: Signature of class, to be used in __init__ method.
        """
        if not PY3:
            StubNode.__init__(self)
        else:
            super().__init__()
        self.name = name  # sig: str
        self.bases = bases  # sig: Sequence[str]
        self.signature = signature  # sig: Optional[str]

    def get_code(self):
        """Get the stub code for this class.

        :sig: () -> List[str]
        :return: Lines of stub code for this class.
        """
        stub = []
        bases = ("(" + ", ".join(self.bases) + ")") if len(self.bases) > 0 else ""
        slots = {"n": self.name, "b": bases}
        if len(self.children) == 0:
            stub.append("class %(n)s%(b)s: ..." % slots)
        else:
            stub.append("class %(n)s%(b)s:" % slots)
            super_code = super().get_code() if PY3 else StubNode.get_code(self)
            for line in super_code:
                stub.append(INDENT + line)
        return stub


def get_aliases(lines):
    """Get the type aliases in the source.

    :sig: (Sequence[str]) -> Dict[str, str]
    :param lines: Lines of the source code.
    :return: Aliases and their their definitions.
    """
    aliases = {}
    for line in lines:
        line = line.strip()
        if len(line) > 0 and line.startswith(SIG_ALIAS):
            _, content = line.split(SIG_ALIAS)
            alias, signature = [t.strip() for t in content.split("=")]
            aliases[alias] = signature
    return aliases


class StubGenerator(ast.NodeVisitor):
    """A transformer that generates stub declarations from a source code."""

    def __init__(self, source):
        """Initialize this stub generator.

        :sig: (str) -> None
        :param source: Source code to generate the stub for.
        """
        self.root = StubNode()  # sig: StubNode

        self.imported_names = OrderedDict()  # sig: OrderedDict[str, str]
        self.defined_types = set()  # sig: Set[str]
        self.required_types = set()  # sig: Set[str]
        self.aliases = OrderedDict()  # sig: OrderedDict[str, str]

        self._parents = [self.root]  # sig: List[StubNode]
        self._code_lines = source.splitlines()  # sig: List[str]

        self.collect_aliases()

        ast_tree = ast.parse(source)
        self.visit(ast_tree)

    def collect_aliases(self):
        """Collect the type aliases in the source.

        :sig: () -> None
        """
        self.aliases = get_aliases(self._code_lines)
        for alias, signature in self.aliases.items():
            _, _, requires = parse_signature(signature)
            self.required_types |= requires
            self.defined_types |= {alias}

    def visit_ImportFrom(self, node):
        """Process a "from x import y" node.

        :sig: (ast.ImportFrom) -> None
        :param node: Node to process.
        """
        line = self._code_lines[node.lineno - 1]
        module_name = line.split("from")[1].split("import")[0].strip()
        for name in node.names:
            self.imported_names[name.name] = module_name

    def visit_Assign(self, node):
        """Process an assignment node.

        :sig: (ast.Assign) -> None
        :param node: Node to process.
        """
        line = self._code_lines[node.lineno - 1]
        if SIG_COMMENT in line:
            line = _RE_COMMENT_IN_STRING.sub("", line)
        if SIG_COMMENT in line:
            _, signature = line.split(SIG_COMMENT)
            _, return_type, requires = parse_signature(signature)
            self.required_types |= requires

            parent = self._parents[-1]
            for var in node.targets:
                if isinstance(var, ast.Name):
                    stub_node = VariableNode(var.id, return_type)
                    parent.add_variable(stub_node)
                if isinstance(var, ast.Attribute) and (var.value.id == "self"):
                    stub_node = VariableNode(var.attr, return_type)
                    parent.parent.add_variable(stub_node)

    def get_function_node(self, node):
        """Process a function node.

        :sig: (Union[ast.FunctionDef, ast.AsyncFunctionDef]) -> FunctionNode
        :param node: Node to process.
        :return: Generated function node in stub tree.
        """
        signature = get_signature(node)

        if signature is None:
            parent = self._parents[-1]
            if isinstance(parent, ClassNode) and (node.name == "__init__"):
                signature = parent.signature

        if signature is not None:
            _logger.debug("parsing signature for %s", node.name)
            param_types, rtype, requires = parse_signature(signature)
            _logger.debug("parameter types: %s", param_types)
            _logger.debug("return type: %s", rtype)
            _logger.debug("required types: %s", requires)
            self.required_types |= requires

            decorators = []
            for d in node.decorator_list:
                if hasattr(d, "id"):
                    decorators.append(d.id)
                elif hasattr(d, "func"):
                    decorators.append(d.func.id)
                elif hasattr(d, "value"):
                    decorators.append(d.value.id + "." + d.attr)

            param_names = [arg.arg if PY3 else arg.id for arg in node.args.args]

            # TODO: only in classes
            if (len(param_names) > 0) and (param_names[0] == "self"):
                param_types.insert(0, "")

            # TODO: only in classes
            if (
                (len(param_names) > 0)
                and (param_names[0] == "cls")
                and ("classmethod" in decorators)
            ):
                param_types.insert(0, "")

            if node.args.vararg is not None:
                param_names.append("*" + (node.args.vararg.arg if PY3 else node.args.vararg))
                param_types.append("")

            if node.args.kwarg is not None:
                param_names.append("**" + (node.args.kwarg.arg if PY3 else node.args.kwarg))
                param_types.append("")

            n_args = len(param_names)

            kwonly_args = getattr(node.args, "kwonlyargs", [])
            if len(kwonly_args) > 0:
                param_names.extend([arg.arg for arg in kwonly_args])

            if len(param_types) != len(param_names):
                raise ValueError("Parameter names and types don't match: " + node.name)

            param_locs = [(a.lineno, a.col_offset) for a in (node.args.args + kwonly_args)]
            param_defaults = {
                bisect(param_locs, (d.lineno, d.col_offset)) - 1 for d in node.args.defaults
            }

            kwonly_defaults = getattr(node.args, "kw_defaults", [])
            for i, d in enumerate(kwonly_defaults):
                if d is not None:
                    param_defaults.add(n_args + i)

            params = [
                (name, type_, i in param_defaults)
                for i, (name, type_) in enumerate(zip(param_names, param_types))
            ]

            if len(kwonly_args) > 0:
                params.insert(n_args, ("*", "", False))

            stub_node = FunctionNode(
                node.name, parameters=params, rtype=rtype, decorators=decorators
            )
            self._parents[-1].add_child(stub_node)

            self._parents.append(stub_node)
            self.generic_visit(node)
            del self._parents[-1]
            return stub_node

    def visit_FunctionDef(self, node):
        """Process a regular function node.

        :sig: (ast.FunctionDef) -> None
        :param node: Node to process.
        """
        node = self.get_function_node(node)
        if node is not None:
            node._async = False

    def visit_AsyncFunctionDef(self, node):
        """Process an async function node.

        :sig: (ast.AsyncFunctionDef) -> None
        :param node: Node to process.
        """
        node = self.get_function_node(node)
        if node is not None:
            node._async = True

    def visit_ClassDef(self, node):
        """Process a class node.

        :sig: (ast.ClassDef) -> None
        :param node: Node to process.
        """
        self.defined_types.add(node.name)

        bases = []
        for n in node.bases:
            base_parts = []
            while True:
                if not isinstance(n, ast.Attribute):
                    base_parts.append(n.id)
                    break
                else:
                    base_parts.append(n.attr)
                n = n.value
            bases.append(".".join(base_parts[::-1]))
        self.required_types |= set(bases)

        signature = get_signature(node)
        stub_node = ClassNode(node.name, bases=bases, signature=signature)
        self._parents[-1].add_child(stub_node)

        self._parents.append(stub_node)
        self.generic_visit(node)
        del self._parents[-1]

    @staticmethod
    def generate_import(module_, names):
        """Generate an import line.

        :sig: (str, Set[str]) -> str
        :param module_: Name of module to import the names from.
        :param names: Names to import.
        :return: Import line in stub code.
        """
        slots = {"m": module_, "n": ", ".join(sorted(names))}
        line = "from %(m)s import %(n)s" % slots
        if len(line) > LINE_LENGTH_LIMIT:
            slots["n"] = INDENT + (",\n" + INDENT).join(sorted(names)) + ","
            line = "from %(m)s import (\n%(n)s\n)" % slots
        return line

    def generate_stub(self):
        """Generate the stub code for this source.

        :sig: () -> str
        :return: Generated stub code.
        """
        needed_types = self.required_types - BUILTIN_TYPES

        needed_types -= self.defined_types
        _logger.debug("defined types: %s", self.defined_types)

        module_vars = {v.name for v in self.root.variables}
        _logger.debug("module variables: %s", module_vars)

        qualified_types = {n for n in needed_types if "." in n}
        qualified_namespaces = {".".join(n.split(".")[:-1]) for n in qualified_types}

        needed_namespaces = qualified_namespaces - module_vars
        needed_types -= qualified_types
        _logger.debug("needed namespaces: %s", needed_namespaces)

        imported_names = set(self.imported_names)
        imported_types = imported_names & (needed_types | needed_namespaces)
        needed_types -= imported_types
        needed_namespaces -= imported_names
        _logger.debug("used imported types: %s", imported_types)

        try:
            typing_mod = __import__("typing")
            typing_types = {n for n in needed_types if hasattr(typing_mod, n)}
            needed_types -= typing_types
            _logger.debug("types from typing module: %s", typing_types)
        except ImportError:
            typing_types = set()
            _logger.warn("typing module not installed")

        if len(needed_types) > 0:
            raise ValueError("Unknown types: " + ", ".join(needed_types))

        out = StringIO()
        started = False

        if len(typing_types) > 0:
            line = self.generate_import("typing", typing_types)
            out.write(line + "\n")
            started = True

        if len(imported_types) > 0:
            if started:
                out.write("\n")
            # preserve the import order in the source file
            for name in self.imported_names:
                if name in imported_types:
                    line = self.generate_import(self.imported_names[name], {name})
                    out.write(line + "\n")
            started = True

        if len(needed_namespaces) > 0:
            if started:
                out.write("\n")
            for module_ in sorted(needed_namespaces):
                out.write("import " + module_ + "\n")
            started = True

        if len(self.aliases) > 0:
            if started:
                out.write("\n")
            for alias, signature in self.aliases.items():
                out.write("%s = %s\n" % (alias, signature))
            started = True

        if started:
            out.write("\n")
        stub_lines = self.root.get_code()
        prev_line = None
        for line in stub_lines:
            if line.startswith("class ") and prev_line and (not prev_line.startswith("class ")):
                out.write("\n")
            if (
                line.startswith("def ")
                and prev_line
                and (prev_line.startswith(" ") or prev_line.startswith("class "))
            ):
                out.write("\n")
            out.write(line + "\n")
            prev_line = line
        return out.getvalue()


def get_stub(source):
    """Get the stub code for a source code.

    :sig: (str) -> str
    :param source: Source code to generate the stub for.
    :return: Generated stub code.
    """
    generator = StubGenerator(source)
    stub = generator.generate_stub()
    return stub


def process_docstring(app, what, name, obj, options, lines):
    """Modify the docstring before generating documentation.

    This will insert type declarations for parameters and return type
    into the docstring, and remove the signature field so that it will
    be excluded from the generated document.
    """
    aliases = getattr(app, "_sigaliases", None)
    if aliases is None:
        if what == "module":
            aliases = get_aliases(inspect.getsource(obj).splitlines())
            app._sigaliases = aliases

    sig_marker = ":" + SIG_FIELD + ":"
    is_class = what in ("class", "exception")

    signature = extract_signature("\n".join(lines))
    if signature is None:
        if not is_class:
            return

        init_method = getattr(obj, "__init__")
        init_doc = init_method.__doc__
        init_lines = init_doc.splitlines()[1:]
        if len(init_lines) > 1:
            init_doc = textwrap.dedent("\n".join(init_lines[1:]))
            init_lines = init_doc.splitlines()
        if sig_marker not in init_doc:
            return

        sig_started = False
        for line in init_lines:
            if line.lstrip().startswith(sig_marker):
                sig_started = True
            if sig_started:
                lines.append(line)
        signature = extract_signature("\n".join(lines))

    if is_class:
        obj = init_method

    param_types, rtype, _ = parse_signature(signature)
    param_names = [p for p in inspect.signature(obj).parameters]

    if is_class and (param_names[0] == "self"):
        del param_names[0]

    # if something goes wrong, don't insert parameter types
    if len(param_names) == len(param_types):
        for name, type_ in zip(param_names, param_types):
            find = ":param %(name)s:" % {"name": name}
            alias = aliases.get(type_)
            if alias is not None:
                type_ = "*%(type)s* :sup:`%(alias)s`" % {"type": type_, "alias": alias}
            for i, line in enumerate(lines):
                if line.startswith(find):
                    lines.insert(i, ":type %(name)s: %(type)s" % {"name": name, "type": type_})
                    break

    if not is_class:
        for i, line in enumerate(lines):
            if line.startswith((":return:", ":returns:")):
                lines.insert(i, ":rtype: " + rtype)
                break

    # remove the signature field
    sig_start = 0
    while sig_start < len(lines):
        if lines[sig_start].startswith(sig_marker):
            break
        sig_start += 1
    sig_end = sig_start + 1
    while sig_end < len(lines):
        if (not lines[sig_end]) or (lines[sig_end][0] != " "):
            break
        sig_end += 1
    for i in reversed(range(sig_start, sig_end)):
        del lines[i]


def setup(app):
    """Register the Sphinx extension."""
    app.connect("autodoc-process-docstring", process_docstring)
    return dict(parallel_read_safe=True)


def main(argv=None):
    """Entry point of the command-line utility.

    :sig: (Optional[List[str]]) -> None
    :param argv: Command line arguments.
    """
    argv = argv if argv is not None else sys.argv
    parser = ArgumentParser(prog="pygenstub")
    parser.add_argument("--version", action="version", version="%(prog)s " + __version__)

    parser.add_argument("source", help="source file")
    parser.add_argument("--debug", action="store_true", help="enable debug messages")
    arguments = parser.parse_args(argv[1:])

    # set debug mode
    if arguments.debug:
        logging.basicConfig(level=logging.DEBUG)
        _logger.debug("running in debug mode")

    with open(arguments.source, mode="r", encoding="utf-8") as f_in:
        code = f_in.read()

    try:
        stub = get_stub(code)
    except RuntimeError as e:
        print(e, file=sys.stderr)
        sys.exit(1)

    if stub != "":
        destination = arguments.source + "i"
        with open(destination, mode="w", encoding="utf-8") as f_out:
            f_out.write("# " + EDIT_WARNING + "\n\n")
            f_out.write(stub)


if __name__ == "__main__":
    main()



from pytest import fixture, raises

import logging
import os
import shutil
import sys

from pkg_resources import get_distribution

import pygenstub


@fixture
def source():
    """Python source code for testing."""
    base_dir = os.path.dirname(__file__)
    src = os.path.join(base_dir, "..", "pygenstub.py")
    dst = "/dev/shm/foo.py" if sys.platform in {"linux", "linux2"} else "foo.py"
    shutil.copy(src, dst)
    yield src, dst

    os.unlink(dst)
    if os.path.exists(dst + "i"):
        os.unlink(dst + "i")


def test_version_should_be_same_as_installed():
    assert get_distribution("pygenstub").version == pygenstub.__version__


def test_cli_help_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        pygenstub.main(argv=["pygenstub", "--help"])
    out, err = capsys.readouterr()
    assert out.startswith("usage: ")


def test_cli_version_should_print_version_number_and_exit(capsys):
    with raises(SystemExit):
        pygenstub.main(argv=["pygenstub", "--version"])
    out, err = capsys.readouterr()
    assert "pygenstub " + pygenstub.__version__ + "\n" in {out, err}


def test_cli_no_input_file_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        pygenstub.main(argv=["pygenstub"])
    out, err = capsys.readouterr()
    assert err.startswith("usage: ")
    assert ("required: source" in err) or ("too few arguments" in err)


def test_cli_unrecognized_arguments_should_print_usage_and_exit(capsys):
    with raises(SystemExit):
        pygenstub.main(argv=["pygenstub", "--foo", "foo.py"])
    out, err = capsys.readouterr()
    assert err.startswith("usage: ")
    assert "unrecognized arguments: --foo" in err


def test_cli_debug_mode_should_print_debug_messages_on_stderr(caplog, source):
    caplog.set_level(logging.DEBUG)
    pygenstub.main(argv=["pygenstub", "--debug", source[1]])
    assert caplog.record_tuples[0][-1] == "running in debug mode"


def test_cli_original_module_should_generate_original_stub(source):
    pygenstub.main(argv=["pygenstub", source[1]])
    with open(source[0] + "i") as src:
        src_stub = src.read()
    with open(source[1] + "i") as dst:
        dst_stub = dst.read()
    assert dst_stub == src_stub



from __future__ import unicode_literals

from pytest import mark, raises

import sys
from io import StringIO

from pygenstub import get_stub


_INDENT = " " * 4


def get_function(
    name, desc="Do foo.", params=None, ptypes=None, rtype=None, decorators=None, body="pass"
):
    code = StringIO()
    if decorators is not None:
        code.write("\n".join(decorators) + "\n")
    pstr = ", ".join(params) if params is not None else ""
    code.write("def %(name)s(%(p)s):\n" % {"name": name, "p": pstr})
    if desc:
        code.write(_INDENT + '"""%(desc)s\n\n' % {"desc": desc})
        tstr = ", ".join(ptypes) if ptypes is not None else ""
        if rtype is not None:
            code.write(_INDENT + ":sig: (%(t)s) -> %(rtype)s\n" % {"t": tstr, "rtype": rtype})
        code.write(_INDENT + '"""\n')
    code.write(_INDENT + body + "\n")
    return code.getvalue()


def get_class(name, bases=None, desc="A foo.", sig=None, methods=None):
    code = StringIO()
    bstr = ("(" + ", ".join(bases) + ")") if bases is not None else ""
    code.write("class %(name)s%(bases)s:\n" % {"name": name, "bases": bstr})
    if desc is not None:
        code.write(_INDENT + '"""%(desc)s\n\n' % {"desc": desc})
        if sig is not None:
            code.write("\n" + _INDENT + ":sig: %(sig)s\n" % {"sig": sig})
    code.write(_INDENT + '"""\n')
    if methods is not None:
        for method in methods:
            for line in method.splitlines():
                code.write(_INDENT + line + "\n")
    else:
        code.write(_INDENT + "pass\n")
    return code.getvalue()


def test_if_no_docstring_then_stub_should_be_empty():
    code = get_function("f", desc="")
    assert get_stub(code) == ""


def test_if_no_sig_then_stub_should_be_empty():
    code = get_function("f")
    assert get_stub(code) == ""


def test_if_returns_none_then_stub_should_return_none():
    code = get_function("f", rtype="None")
    assert get_stub(code) == "def f() -> None: ...\n"


def test_if_returns_builtin_then_stub_should_return_builtin():
    code = get_function("f", rtype="int")
    assert get_stub(code) == "def f() -> int: ...\n"


def test_if_returns_from_imported_then_stub_should_include_import():
    code = "from x import A\n"
    code += "\n\n" + get_function("f", rtype="A")
    assert get_stub(code) == "from x import A\n\ndef f() -> A: ...\n"


def test_stub_should_exclude_unused_import():
    code = "from x import A, B\n"
    code += "\n\n" + get_function("f", rtype="A")
    assert get_stub(code) == "from x import A\n\ndef f() -> A: ...\n"


# def test_stub_should_include_all_used_imports():
#     code = "from x import A, B\n"
#     code += "\n\n" + get_function("f", rtype="A")
#     code += "\n\n" + get_function("g", rtype="B")
#     assert get_stub(code) == "from x import A, B\n\ndef f() -> A: ...\ndef g() -> B: ...\n"


def test_if_returns_imported_qualified_then_stub_should_include_import():
    code = "import x\n"
    code += "\n\n" + get_function("f", rtype="x.A")
    assert get_stub(code) == "import x\n\ndef f() -> x.A: ...\n"


def test_if_returns_from_imported_qualified_then_stub_should_include_import():
    code = "from x import y\n"
    code += "\n\n" + get_function("f", rtype="y.A")
    assert get_stub(code) == "from x import y\n\ndef f() -> y.A: ...\n"


def test_if_returns_relative_imported_qualified_then_stub_should_include_import():
    code = "from . import x\n"
    code += "\n\n" + get_function("f", rtype="x.A")
    assert get_stub(code) == "from . import x\n\ndef f() -> x.A: ...\n"


def test_if_returns_unimported_qualified_then_stub_should_generate_import():
    code = get_function("f", rtype="x.y.A")
    assert get_stub(code) == "import x.y\n\ndef f() -> x.y.A: ...\n"


def test_if_returns_imported_typing_then_stub_should_include_import():
    code = "from typing import List\n"
    code += "\n\n" + get_function("f", rtype="List")
    assert get_stub(code) == "from typing import List\n\ndef f() -> List: ...\n"


def test_if_returns_unimported_typing_then_stub_should_generate_import():
    code = get_function("f", rtype="List")
    assert get_stub(code) == "from typing import List\n\ndef f() -> List: ...\n"


def test_if_returns_qualified_typing_then_stub_should_return_qualified_typing():
    code = get_function("f", rtype="List[str]")
    assert get_stub(code) == "from typing import List\n\ndef f() -> List[str]: ...\n"


def test_if_returns_multiple_typing_then_stub_should_generate_multiple_import():
    code = get_function("f", rtype="Dict[str, Any]")
    assert get_stub(code) == "from typing import Any, Dict\n\ndef f() -> Dict[str, Any]: ...\n"


def test_if_returns_unknown_type_should_raise_error():
    code = get_function("f", rtype="Foo")
    with raises(ValueError) as e:
        get_stub(code)
    assert "Unknown types: Foo" in str(e)


def test_if_one_param_then_stub_should_have_one_param():
    code = get_function("f", params=["i"], ptypes=["int"], rtype="None")
    assert get_stub(code) == "def f(i: int) -> None: ...\n"


def test_if_multiple_params_then_stub_should_have_multiple_params():
    code = get_function("f", params=["i", "s"], ptypes=["int", "str"], rtype="None")
    assert get_stub(code) == "def f(i: int, s: str) -> None: ...\n"


def test_if_param_type_imported_then_stub_should_include_import():
    code = "from x import A\n"
    code += "\n\n" + get_function("f", params=["a"], ptypes=["A"], rtype="None")
    assert get_stub(code) == "from x import A\n\ndef f(a: A) -> None: ...\n"


def test_if_param_type_from_imported_then_stub_should_include_import():
    code = "from x import A\n"
    code += "\n\n" + get_function("f", params=["a"], ptypes=["A"], rtype="None")
    assert get_stub(code) == "from x import A\n\ndef f(a: A) -> None: ...\n"


def test_if_param_type_imported_qualified_then_stub_should_include_import():
    code = "import x\n"
    code += "\n\n" + get_function("f", params=["a"], ptypes=["x.A"], rtype="None")
    assert get_stub(code) == "import x\n\ndef f(a: x.A) -> None: ...\n"


def test_if_param_type_from_imported_qualified_then_stub_should_include_import():
    code = "from x import y\n"
    code += get_function("f", params=["a"], ptypes=["y.A"], rtype="None")
    assert get_stub(code) == "from x import y\n\ndef f(a: y.A) -> None: ...\n"


def test_if_param_type_relative_imported_qualified_then_stub_should_include_import():
    code = "from . import x\n"
    code += "\n\n" + get_function("f", params=["a"], ptypes=["x.A"], rtype="None")
    assert get_stub(code) == "from . import x\n\ndef f(a: x.A) -> None: ...\n"


def test_if_param_type_unimported_qualified_then_stub_should_generate_import():
    code = get_function("f", params=["a"], ptypes=["x.y.A"], rtype="None")
    assert get_stub(code) == "import x.y\n\ndef f(a: x.y.A) -> None: ...\n"


def test_if_param_type_imported_typing_then_stub_should_include_import():
    code = "from typing import List\n"
    code += "\n\n" + get_function("f", params=["l"], ptypes=["List"], rtype="None")
    assert get_stub(code) == "from typing import List\n\ndef f(l: List) -> None: ...\n"


def test_if_param_type_unimported_typing_then_stub_should_include_import():
    code = get_function("f", params=["l"], ptypes=["List"], rtype="None")
    assert get_stub(code) == "from typing import List\n\ndef f(l: List) -> None: ...\n"


def test_if_param_type_qualified_typing_then_stub_should_include_qualified_typing():
    code = get_function("f", params=["ls"], ptypes=["List[str]"], rtype="None")
    assert get_stub(code) == "from typing import List\n\ndef f(ls: List[str]) -> None: ...\n"


def test_if_param_type_multiple_typing_then_stub_should_include_multiple_import():
    code = get_function("f", params=["d"], ptypes=["Dict[str, Any]"], rtype="None")
    assert (
        get_stub(code)
        == "from typing import Any, Dict\n\ndef f(d: Dict[str, Any]) -> None: ...\n"
    )


def test_if_param_type_unknown_should_raise_error():
    code = get_function("f", params=["i"], ptypes=["Foo"], rtype="None")
    with raises(ValueError) as e:
        get_stub(code)
    assert "Unknown types: Foo" in str(e)


def test_if_param_has_default_then_stub_should_include_ellipses():
    code = get_function("f", params=["i=0"], ptypes=["Optional[int]"], rtype="None")
    assert (
        get_stub(code)
        == "from typing import Optional\n\ndef f(i: Optional[int] = ...) -> None: ...\n"
    )


def test_stub_should_ignore_varargs_type():
    code = get_function("f", params=["i", "*args"], ptypes=["int"], rtype="None")
    assert get_stub(code) == "def f(i: int, *args) -> None: ...\n"


def test_stub_should_ignore_kwargs_type():
    code = get_function("f", params=["i", "**kwargs"], ptypes=["int"], rtype="None")
    assert get_stub(code) == "def f(i: int, **kwargs) -> None: ...\n"


def test_stub_should_ignore_vararg_and_kwargs_types():
    code = get_function("f", params=["i", "*args", "**kwargs"], ptypes=["int"], rtype="None")
    assert get_stub(code) == "def f(i: int, *args, **kwargs) -> None: ...\n"


@mark.skipif(sys.version_info < (3, 0), reason="syntax introduced in py3")
def test_stub_should_honor_kwonly_args():
    code = get_function("f", params=["i", "*", "j"], ptypes=["int", "int"], rtype="None")
    assert get_stub(code) == "def f(i: int, *, j: int) -> None: ...\n"


@mark.skipif(sys.version_info < (3, 0), reason="syntax introduced in py3")
def test_stub_should_honor_kwonly_args_with_default():
    code = get_function(
        "f", params=["i", "*", "j=0"], ptypes=["int", "Optional[int]"], rtype="None"
    )
    assert (
        get_stub(code)
        == "from typing import Optional\n\ndef f(i: int, *, j: Optional[int] = ...) -> None: ...\n"
    )


def test_missing_types_should_raise_error():
    code = get_function("f", params=["i", "j"], ptypes=["int"], rtype="None")
    with raises(ValueError) as e:
        get_stub(code)
    assert "Parameter names and types don't match: " in str(e)


def test_extra_types_should_raise_error():
    code = get_function("f", params=["i"], ptypes=["int", "int"], rtype="None")
    with raises(ValueError) as e:
        get_stub(code)
    assert "Parameter names and types don't match: " in str(e)


def test_if_function_decorated_unknown_then_stub_should_ignore():
    code = get_function("f", rtype="None", decorators=["@foo"])
    assert get_stub(code) == "def f() -> None: ...\n"


def test_if_function_decorated_callable_unknown_then_stub_should_ignore():
    code = get_function("f", rtype="None", decorators=["@foo()"])
    assert get_stub(code) == "def f() -> None: ...\n"


def test_stub_should_include_empty_class():
    code = get_class("C")
    assert get_stub(code) == "class C: ...\n"


def test_method_stub_should_include_self():
    method = get_function("m", params=["self"], rtype="None")
    code = get_class("C", methods=[method])
    assert get_stub(code) == "class C:\n    def m(self) -> None: ...\n"


def test_method_stub_should_include_params():
    method = get_function("m", params=["self", "i"], ptypes=["int"], rtype="None")
    code = get_class("C", methods=[method])
    assert get_stub(code) == "class C:\n    def m(self, i: int) -> None: ...\n"


def test_if_base_builtin_then_stub_should_include_base():
    code = get_class("C", bases=["dict"])
    assert get_stub(code) == "class C(dict): ...\n"


def test_if_base_from_imported_then_stub_should_include_import():
    code = "from x import A\n"
    code += "\n\n" + get_class("C", bases=["A"])
    assert get_stub(code) == "from x import A\n\nclass C(A): ...\n"


def test_if_base_imported_qualified_then_stub_should_include_import():
    code = "import x\n"
    code += "\n\n" + get_class("C", bases=["x.A"])
    assert get_stub(code) == "import x\n\nclass C(x.A): ...\n"


def test_if_base_from_imported_qualified_then_stub_should_include_import():
    code = "from x import y\n"
    code += "\n\n" + get_class("C", bases=["y.A"])
    assert get_stub(code) == "from x import y\n\nclass C(y.A): ...\n"


def test_if_base_unimported_qualified_then_stub_should_generate_import():
    code = get_class("C", bases=["x.y.A"])
    assert get_stub(code) == "import x.y\n\nclass C(x.y.A): ...\n"


def test_if_base_relative_imported_qualified_then_stub_should_include_import():
    code = "from . import x\n"
    code += "\n\n" + get_class("C", bases=["x.A"])
    assert get_stub(code) == "from . import x\n\nclass C(x.A): ...\n"


def test_class_sig_should_be_moved_to_init_method():
    method = get_function("__init__", params=["self", "x"], rtype=None)
    code = get_class("C", sig="(str) -> int", methods=[method])
    assert get_stub(code) == "class C:\n    def __init__(self, x: str) -> int: ...\n"


def test_class_sig_should_not_overwrite_existing_init_sig():
    method = get_function("__init__", params=["self", "x"], ptypes=["int"], rtype="None")
    code = get_class("C", sig="(str) -> int", methods=[method])
    assert get_stub(code) == "class C:\n    def __init__(self, x: int) -> None: ...\n"


def test_if_method_decorated_unknown_then_stub_should_ignore():
    method = get_function("m", params=["self"], rtype="None", decorators=["@foo"])
    code = get_class("C", methods=[method])
    assert get_stub(code) == "class C:\n    def m(self) -> None: ...\n"


def test_if_method_decorated_callable_unknown_then_stub_should_ignore():
    method = get_function("m", params=["self"], rtype="None", decorators=["@foo()"])
    code = get_class("C", methods=[method])
    assert get_stub(code) == "class C:\n    def m(self) -> None: ...\n"


def test_if_method_decorated_staticmethod_then_stub_should_include_decorator():
    method = get_function("m", rtype="None", decorators=["@staticmethod"])
    code = get_class("C", methods=[method])
    assert get_stub(code) == "class C:\n    @staticmethod\n    def m() -> None: ...\n"


def test_if_method_decorated_classmethod_then_stub_should_include_decorator():
    method = get_function("m", params=["cls"], rtype="None", decorators=["@classmethod"])
    code = get_class("C", methods=[method])
    assert get_stub(code) == "class C:\n    @classmethod\n    def m(cls) -> None: ...\n"


def test_if_method_decorated_property_then_stub_should_include_decorator():
    method = get_function("m", params=["self"], rtype="None", decorators=["@property"])
    code = get_class("C", methods=[method])
    assert get_stub(code) == "class C:\n    @property\n    def m(self) -> None: ...\n"


def test_if_method_decorated_property_setter_then_stub_should_include_decorator():
    method = get_function("m", params=["self"], rtype="None", decorators=["@x.setter"])
    code = get_class("C", methods=[method])
    assert get_stub(code) == "class C:\n    @x.setter\n    def m(self) -> None: ...\n"


def test_module_variable_type_comment_builtin_should_be_ellipsized():
    code = "n = 42  # sig: int\n"
    assert get_stub(code) == "n = ...  # type: int\n"


def test_module_variable_type_comment_from_imported_should_include_import():
    code = "from x import A\n\nn = 42  # sig: A\n" ""
    assert get_stub(code) == "from x import A\n\nn = ...  # type: A\n"


def test_module_variable_type_comment_imported_qualified_should_include_import():
    code = "import x\n\nn = 42  # sig: x.A\n" ""
    assert get_stub(code) == "import x\n\nn = ...  # type: x.A\n"


def test_module_variable_type_comment_relative_qualified_should_include_import():
    code = "from . import x\n\nn = 42  # sig: x.A\n" ""
    assert get_stub(code) == "from . import x\n\nn = ...  # type: x.A\n"


def test_module_variable_type_comment_unimported_qualified_should_include_import():
    code = "n = 42  # sig: x.y.A\n" ""
    assert get_stub(code) == "import x.y\n\nn = ...  # type: x.y.A\n"


def test_get_stub_comment_instance_variable():
    method = get_function("m", params=["self"], rtype="None", body="self.a = 42  # sig: int")
    code = get_class("C", methods=[method])
    assert (
        get_stub(code) == "class C:\n    a = ...  # type: int\n    def m(self) -> None: ...\n"
    )


def test_stub_should_use_alias_comment():
    code = "# sigalias: B = int\n\nn = 42  # sig: B\n" ""
    assert get_stub(code) == "B = int\n\nn = ...  # type: B\n"


def test_stub_should_exclude_function_without_sig():
    code = get_function("f", rtype="None")
    code += "\n\n" + get_function("g", desc="")
    assert get_stub(code) == "def f() -> None: ...\n"


def test_get_stub_typing_import_should_come_first():
    code = "from x import A\n"
    code += "\n\n" + get_function("f", params=["a", "l"], ptypes=["A", "List"], rtype="None")
    assert (
        get_stub(code)
        == "from typing import List\n\nfrom x import A\n\ndef f(a: A, l: List) -> None: ...\n"
    )



# -*- coding: utf-8 -*-
from distutils.core import setup

modules = \
['pygenstub']
install_requires = \
['docutils>=0.14,<0.15']

entry_points = \
{'console_scripts': ['pygenstub = pygenstub:main']}

setup_kwargs = {
    'name': 'pygenstub',
    'version': '1.2.3',
    'description': 'Python stub file generator.',
    'long_description': 'Copyright (C) 2016-2019 H. Turgut Uyar <uyar@tekir.org>\n\npygenstub is a utility for generating stub files from docstrings\nin source files. It takes a source file as input and creates a stub file\nwith the same base name and the ``.pyi`` extension.\n\n:PyPI: https://pypi.org/project/pygenstub/\n:Repository: https://github.com/uyar/pygenstub\n:Documentation: https://pygenstub.readthedocs.io/\n\nIf the docstring of a function includes a **sig** field, the value of that\nfield will be used to generate a prototype by matching the types to the\nparameters in the same order. For example, for the code given below:\n\n.. code-block:: python\n\n   def foo(a, b):\n       """Do foo.\n\n       :sig: (int, str) -> None\n       """\n\npygenstub will generate the following stub:\n\n.. code-block:: python\n\n   def foo(a: int, b: str) -> None: ...\n\npygenstub consists of a single module which itself contains signatures.\nYou can see the `source code`_ and the autogenerated `stub file`_\nas an example.\n\nUsage\n-----\n\npygenstub has been tested with Python 2.7, Python 3.4+, and compatible\nversions of PyPy. You can install the latest version using ``pip``::\n\n  pip install pygenstub\n\nInstallation creates a script named ``pygenstub`` which can be used\nas follows::\n\n  pygenstub foo.py\n\nThis command will generate the file ``foo.pyi`` in the same directory\nas the input file. If the output file already exists, it will be overwritten.\n\nSphinx autodoc support\n----------------------\n\nIf pygenstub is activated as a Sphinx extension (after autodoc), it will insert\ntype comments for parameters and return values in the docstring. It will also\nremove the signature fields to exclude them from the output:\n\n.. code-block:: python\n\n   extensions = [\n       \'sphinx.ext.autodoc\',\n       \'pygenstub\'\n   ]\n\nAs an example of the output, you can see the `API documentation`_\nfor pygenstub itself.\n\n.. _source code: https://github.com/uyar/pygenstub/blob/master/pygenstub.py\n.. _stub file: https://github.com/uyar/pygenstub/blob/master/pygenstub.pyi\n.. _API documentation: https://pygenstub.readthedocs.io/en/latest/api.html\n',
    'author': 'H. Turgut Uyar',
    'author_email': 'uyar@tekir.org',
    'url': 'https://github.com/uyar/pygenstub',
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
}


setup(**setup_kwargs)
